vec3 cloudLayer2Wind    = normalize(vec3(0.75, 0.1, -0.05)) * cloudTick;

float cloudLayer2Cirrocumulus(vec3 pos, float altitude, float coverageFade) {
    const float maxalt  = cloudLayer2PlanarMaxY;
    const float minalt  = cloudLayer2PlanarMinY;

    if (altitude > maxalt || altitude < minalt) return 0.0;

    const float depth   = maxalt - minalt;


    //  coverage section

    float erodeLow      = 1.0 - sstep(altitude, minalt, minalt + (depth * 0.4));
    float erodeHigh     = sstep(altitude, minalt + (depth * 0.7), maxalt);
    float fadeLow       = sstep(altitude, minalt, minalt + (depth * 0.3));
    float fadeHigh      = 1.0 - sstep(altitude, minalt + (depth * 0.85), maxalt);

    //if (max(fadeLow, fadeHigh) < 1e-2) return 0.0;

    float localCoverage = readNoise2D((pos.xz + cloudLayer2Wind.xz * 3e-2) * 5e-3 - 0.725).b;
        localCoverage   = sstep(localCoverage - cloudLayer2data.y, 0.35, 0.65);

    //return localCoverage * 10;

    if (localCoverage > 0.995) return 0.0;

    const float coverageBias  = 0.37;

    vec4 nCov           = readNoise2D(pos.xz * 14e-2);

    float coverage      = nCov.z + cloudLayer2data.x;
        coverage        = (coverage - coverageBias) * rcp(saturate(1.0 - coverageBias)) - (localCoverage * 0.5);
        coverage       *= (1.0  - localCoverage);
        coverage        = (coverage - (coverageFade * 0.2)) * (1.0 - coverageFade);

        coverage        = (coverage * fadeLow) - (erodeLow * 0.6);
        coverage        = (coverage * fadeHigh) - (erodeHigh * (0.5 + localCoverage * 0.1));
        coverage        = saturate(coverage * 1.25);
    
    if (coverage < 36e-3) return 0.0;


    //  shape detail section

    float shape         = coverage;
    float slope         = sqrt(saturate(1.0 - shape));

    if (slope > 1e-8) {
        float fadeWorley = sstep(altitude, minalt, minalt + (depth * 0.33)) * 0.5 + 0.5;

        vec3 n3D        = noise_3d(pos * 1.5 + shape * 0.4);
        vec2 nd         = vec2(n3D.x + n3D.y * 0.4, 1.4);

            shape      -= (1.0 - (nd.x * rcp(nd.y))) * 0.3 * slope;
        
        if (shape < 0.0) return 0.0;

        shape *= (max(shape, 0.0));
    }

    return max(shape * 0.07 * cloudLayer2data.z, 0.0);
}
float cloudLayer2Cirrus(vec3 pos, float altitude, float coverageFade, float auxNoise) {
    const float maxalt  = cloudLayer2PlanarMaxY;
    const float minalt  = cloudLayer2PlanarMinY;

    if (altitude > maxalt || altitude < minalt) return 0.0;

    const float depth   = maxalt - minalt;


    //  coverage section

    float erodeLow      = 1.0 - sstep(altitude, minalt, minalt + (depth * 0.4));
    float erodeHigh     = sstep(altitude, minalt + (depth * 0.8), maxalt);
    float fadeLow       = sstep(altitude, minalt, minalt + (depth * 0.25));
    float fadeHigh      = 1.0 - sstep(altitude, minalt + (depth * 0.8), maxalt);

    //if (max(fadeLow, fadeHigh) < 1e-2) return 0.0;

    float localCoverage = readNoise2D((pos.xz + cloudLayer2Wind.xz * 3e-2) * 3e-4 - 0.315).x;
        localCoverage   = sqr(sstep(localCoverage - cloudLayer2data.y, 0.1, 0.8));

    //return localCoverage;

    if (localCoverage > 0.995) return 0.0;

    const float coverageBias  = 0.44;

    pos.yz += auxNoise * 3.0;

    vec4 nCov           = readNoise2D((pos.xz * vec2(0.4, 1.0)) * 3e-2);

    float coverage      = nCov.z + cloudLayer2data.x;
        coverage        = (coverage - coverageBias) * rcp(saturate(1.0 - coverageBias)) - (localCoverage * 0.3);
        coverage       *= (1.0  - localCoverage);
        coverage        = (coverage - (coverageFade * 0.3)) * (1.0 - coverageFade);

        coverage        = (coverage * fadeLow) - (erodeLow * 0.6);
        coverage        = (coverage * fadeHigh) - (erodeHigh * (0.5));
        coverage        = saturate(coverage * 1.0);
    
    if (coverage < 36e-3) return 0.0;


    //  shape detail section

    float shape         = coverage;
    float slope         = sqrt(1.0 - saturate(abs(shape - 0.5) * 2.0));

    if (slope > 1e-8) {
            pos.z      *= 0.35;

            pos.x      += shape * 2.0;
            pos.z      += shape * 1.7;

            shape      -= value_3d(pos * 10.0) * 0.20 * slope; 
        
        if (shape < 0.0) return 0.0;

            slope       = (saturate(1.0 - shape));

        if (slope > 1e-8) {
            pos.xy     += shape * 0.45;

            shape      -= value_3d(pos * 32.0) * 0.08 * slope;

            shape *= max(shape, 0.0);
        }
    }

    return max(shape * 0.13 * cloudLayer2data.z, 0.0);
}