
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/


#define sun_illum_mult 1.0  //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.5 3.0 3.5 4.0 5.0 6.0 7.0 8.0 9.0 10.0]
#define moon_illum_mult 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.5 3.0 3.5 4.0 5.0 6.0 7.0 8.0 9.0 10.0 15.0 20.0 25.0 30.0 35.0 40.0 50.0]


//Planet atmosphere constants
const float airNumberDensity = 2.4690778e25;
const float ozoneNumberDensity = airNumberDensity * exp(-35e3 / 8e3) * 2.80475 * 4e-6;

const float planetRadius = 6371e3;
const float atmosphereHeight = 110e3;

const vec2 scaleHeights = vec2(8.5e3, 1.2e3);

const float mieG = 0.76*atmos_mieGmult;

const vec3 ozoneCrossSection = vec3(4.51103766177301E-21, 3.2854797958699E-21, 1.96774621921165E-22) * 0.0001;


//const vec3 rayleighCoefficient1 = vec3(5.847e-6, 1.35e-5, 3.31e-5)*atmos_rayCoeff;
//const vec3 rayleighCoefficient2  = vec3(6.4290568167e-6, 1.0866371330e-5, 2.4844733008e-5) * atmos_rayCoeff;
//const vec3 rayleighCoefficient  = rayleighCoefficient1 * 0.5 + rayleighCoefficient2 * 0.5;
//const vec3 rayleighCoefficient = vec3(6.433377384678407e+24, 1.0873673940138444e+25, 2.4861429602679963e+25)*atmos_rayCoeff*9.993284137187039e-31;
//const vec3 rayleighCoefficient  = vec3(6.1380284083e-6, 1.2183185665e-5, 2.8972366504e-5) * _sRGB_AP1 * atmos_rayCoeff;
//const vec3 rayleighCoefficient = vec3(5.847e-6, 1.35e-5, 3.31e-5) * _sRGB_AP1 * atmos_rayCoeff;
const vec3 rayleighCoefficient = vec3(9.73656933e-06, 1.32262287e-05, 3.03837004e-05) * atmos_rayCoeff;
const vec3 ozoneCoefficient = (ozoneNumberDensity * ozoneCrossSection) * _sRGB_AP1;
//const vec3 mieCoefficient = vec3(5.5e-6)*2.0*atmos_mieCoeff;
const vec3 mieCoefficient = vec3(5e-6)*atmos_mieCoeff;
const vec3 mistCoefficient = vec3(4e-5);    //this is used for fog and the value is a placeholder for something more accurate

const vec2 inverseScaleHeights = 1.0 / (scaleHeights);
const vec2 scaledPlanetRadius = planetRadius * inverseScaleHeights;
const float atmosphereRadius        = planetRadius + atmosphereHeight;
const float atmosphereLowerLimit = planetRadius + -10e3;

const mat2x3 scatteringCoefficients = mat2x3(rayleighCoefficient, mieCoefficient);
const mat3x3 extinctionCoefficients = mat3x3(rayleighCoefficient, mieCoefficient * 1.11, ozoneCoefficient);

const mat3x3 fogScatteringCoefficients = mat3x3(rayleighCoefficient, mieCoefficient, mistCoefficient);
const mat3x3 fogExtinctionCoefficients = mat3x3(rayleighCoefficient, mieCoefficient * 1.11, mistCoefficient);


/*
    These are the constants for the celestial bodies.
    Some of these constants aren't completely accurate because we don't have enough precision in an RGBA16F buffer for them to be correct.
*/
const float sunRadius = 1392082.56;
const float sunDist = 149.6e6;

const float sunAngularRadius = sunRadius / (sunDist * 2.0);

const float moonRadius = 1737e3;
/*
    Pretty sure this can be either 379700e3 or 384400e3, since it changes throughout the year.
*/
const float moonSemiMajorAxis = 384400e3;

const float moonAngularRadius = moonRadius / moonSemiMajorAxis;

//units in lux * 1e-2
const vec3 sunIlluminance = (vec3(1.0, 0.949, 0.937) * _sRGB_AP1) * lux(100000) * sun_illum_mult;
const vec3 sunLuminance = sunIlluminance / coneAngleToSolidAngle(sunAngularRadius);

const vec3 moonColor = vec3(0.136) * 10 * moon_illum_mult;
const vec3 moonLuminance = moonColor * sunIlluminance / pi;
const vec3 moonIlluminance = moonLuminance * coneAngleToSolidAngle(moonAngularRadius);