
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



vec3 physicalSun(in vec3 sceneDir) {
    float cosTheta = dot(sceneDir, sunVector);
    float centerToEdge = clamp(acos(cosTheta) / sunAngularRadius, 0.0, 1.0);
    float cosSunRadius = cos(sunAngularRadius);
    if(cosTheta < cosSunRadius) return vec3(0.0);

    vec3 a = vec3 (0.397 ,  0.503,  0.652);

    float  mu = sqrt(1.0 - centerToEdge*centerToEdge);
    vec3 factor = 1.0 - (1.0 - pow(vec3(mu), a));
    vec3 finalLuminance = sunLuminance * factor;

    return finalLuminance;
}

vec3 physicalMoon(in vec3 sceneDir) {
	vec3 viewPos = vec3(0.0, planetRadius + eyeAltitude, 0.0);
    vec2 dists = rsi(-moonVector, sceneDir, moonAngularRadius);

    vec3 moonNormal = normalize(sceneDir*dists.x - moonVector);

    vec3 diffuse = orenNayar(0.3, moonColor, -sceneDir, sunVector, moonNormal);

    //This should be using the sunIlluminance iirc, but it makes the moon "too" bright in comparison to the sun due to the precision we have, so we are using the moonIlluminance*200 instead.
	return dists.y > 0.0 ? (diffuse * (moonIlluminance*200.0)) : vec3(0.0);
}