/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

float vcloud_tick   = frameTimeCounter * 1e-1;


/* ------ constants ------ */

//#define vcloud_steps 100        //[60 80 100 120 140 160 180 200 220 240 260 280 300]
#define vcloud_quality 1.0      //[0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.2 1.4 1.6 1.8 2.0 2.2 2.4 2.6 2.8 3.0]
#define vcloud_direct_steps 6   //[4 5 6 7 8 9 10 12]
#define vcloud_skylight_steps 4 //[2 3 4 5 6 7 8]
#define vcloud_thresh 0.075     //[0.2 0.15 0.1 0.075 0.05 0.025 0.01]
#define vcloud_shapeq 1         //[0 1 2]
#define vcloud_lowlayer
#define vcloud_midlayer
#define vcloud_highlayer 1      //[0 1 2]

const float vcloud_minalt   = 7e2;
const float vcloud_lowmax   = 3e3;
const float vcloud_midmax   = 8e3;
const float vcloud_maxalt   = 12e3;

const float pcloud_cirrusalt    = 8.65e3;

const float vcloud_lowdepth = vcloud_lowmax - vcloud_minalt;
const float vcloud_middepth = vcloud_midmax - vcloud_lowmax;
const float vcloud_highdepth = vcloud_maxalt - vcloud_midmax;

const float vcloud_depth    = vcloud_maxalt - vcloud_minalt;

const float vcloud_clip     = 110e3;

#define vcloud_lowsize 14e-4
#define vcloud_midsize 7e-4
#define vcloud_highsize 5e-4

/* setup altitudes for different cloud types */

#define vcloud_cumulus_min 7e2
#define vcloud_cumulus_max 3e3
#define vcloud_stratocumulus_min 16e2
#define vcloud_stratocumulus_max 27e2

#define vcloud_altocumulus_min 3e3
#define vcloud_altocumulus_max 6e3
#define vcloud_altostratus_min 5.5e3
#define vcloud_altostratus_max 8e3

#define vcloud_cirrus_min 8e3
#define vcloud_cirrus_max 12e3
#define vcloud_cirrocumuli_min 8e3
#define vcloud_cirrocumuli_max 10e3

#ifdef vcloud_dynamic_type
    uniform vec4 vcloud_lowlayer_data;
    uniform vec4 vcloud_midlayer_data;
    uniform vec4 vcloud_highlayer_data;
#else
    vec4 vcloud_lowlayer_data = vec4(0.0 - wetness * 0.15, 0.0 - wetness * 0.12, 1.0, 0.0);
    vec4 vcloud_midlayer_data = vec4(0.0 - wetness * 0.15, 0.0 - wetness * 0.1, 1.0, 0.0);
    vec4 vcloud_highlayer_data = vec4(0.0, 0.0, 1.0, 0.0);
#endif

const vec3 vcloud_lowlayer_winddir  = normalize(vec3(0.55, -0.05, 0.45));
const vec3 vcloud_midlayer_winddir  = normalize(vec3(0.55, -0.15, 0.25));
const vec3 vcloud_highlayer_winddir = normalize(vec3(0.75, 0.1, -0.05));


/* ------ functions ------ */

#ifdef PD_CLOUDRENDER
float cloud_mie_dumb(float cos_theta, float g) {
    float sqG   = sqr(g);
    float a     = (1.0-sqG) / (2.0 + sqG);
    float b     = (1.0 + sqr(cos_theta)) / (1.0 + sqG - 2.0*g*cos_theta);

    return max(1.5 * a*b + g*cos_theta, 0.0)*rcp(pi);
}
float cloud_mie(float x, float g) {
    float mie   = 1.0 + sqr(g) - 2.0*g*x;
        mie     = (1.0 - sqr(g)) / ((4.0*pi) * mie*(mie*0.5+0.5));
    return mie;
}
float cloud_phase(float cos_theta, float g) {
    float a     = cloud_mie(cos_theta, 0.44*g)+cloud_mie_dumb(cos_theta, 0.89*g)*0.75;
    float b     = cloud_mie(cos_theta, -0.25*g)*1.4*cloudBackscattering;

    return mix(a, b, 0.38);   //kinda looks better to me with a lil min value
}
#endif

vec2 rotate_coord(vec2 pos, const float angle) {
    return vec2(cos(angle)*pos.x + sin(angle)*pos.y, 
                cos(angle)*pos.y - sin(angle)*pos.x);
}

float fbm(vec3 pos, vec3 offset, const float persistence, const float scale, const int octaves) {
    float n     = 0.0;
    float a     = 1.0;
    vec3 shift  = offset;

    for (int i = 0; i<octaves; ++i) {
        n      += value_3d(pos + shift)*a;
        pos.xz  = rotate_coord(pos.xz, pi*0.33);
        pos    *= scale;
        a      *= persistence;
    }

    return n;
}


/* ------ cloud shape ------ */

float vcloud_shape_lowlayer(vec3 pos, float altitude, float lod_dist, out vec2 powder_k) {
    vec3 wind       = vec3(vcloud_tick) * vcloud_lowlayer_winddir;
    pos            *= vcloud_lowsize;
    pos            += wind*0.1;

    float fadenoise  = noise_2d(pos.xz * 0.003).b + vcloud_lowlayer_data.w;
        fadenoise   += vcloud_lowlayer_override;

    float stratocumuli_fade = sqr(sstep(fadenoise, 0.45, 0.65));
    float cov_fade  = sstep(fadenoise, 0.2, 0.55) * (1.0 - sstep(fadenoise, 0.55, 0.80));

    float minalt        = mix(vcloud_cumulus_min, vcloud_stratocumulus_min, stratocumuli_fade);
    float maxalt        = mix(vcloud_cumulus_max, vcloud_stratocumulus_max, stratocumuli_fade);
    float depth         = maxalt - minalt;

    if (altitude > maxalt || altitude < minalt) return 0.0;


    /* ------ coverage section ------ */

    float erode_low     = 1.0-sstep(altitude, minalt, minalt+depth*0.13);
    float erode_high    = sstep(altitude, minalt+depth*0.16, maxalt);
    float fade_low      = sstep(altitude, minalt, minalt+depth*0.075);
    float fade_high     = 1.0-sstep(altitude, minalt+depth*0.65, maxalt);

    if (max(fade_low, fade_high) < 0.01) return 0.0;

    float local_cov = noise_2d(pos.xz*0.0013 - wind.xz * 0.0001).x;
        local_cov   = sstep(local_cov+vcloud_lowlayer_data.y, 0.42, 0.7);

    if (local_cov > 0.995) return 0.0;

    float coverage_bias = 0.41 + stratocumuli_fade * 0.025 + vcloud_lowlayer_data.x;
        //coverage_bias += pow3(cov_fade) * 0.08;
        //coverage_bias = -0.5;

    float coverage  = noise_2d(pos.xz*0.03).b;
        coverage    = (coverage - coverage_bias) * rcp(1.0 - saturate(coverage_bias));

        coverage   -= local_cov*0.5;
        coverage   *= (1.0-local_cov);

        coverage   *= fade_low;
        coverage   *= fade_high;
        coverage   -= erode_low*0.2;
        coverage   -= erode_high*0.75;

        //coverage    = stratocumuli_fade;

        coverage    = saturate(coverage * 1.25);

    if (coverage < 0.036) return 0.0;


    /* ------ shape detail section ------ */

    float fade_worley   = sstep(altitude, minalt, minalt+depth*0.33)*0.5+0.5;

    float fade_top      = sstep(altitude, minalt+depth*0.2, minalt+depth*0.65);
        //fade_top       += ;

    float fade_density  = 0.01 + sstep(altitude, minalt, minalt+depth*0.2)*0.08;
        fade_density   += sstep(altitude, minalt+depth*0.1, minalt+depth*0.55)*0.3;
        fade_density   += fade_top * 1.2;
        fade_density   += sstep(altitude, minalt+depth*0.4, maxalt-depth*0.15)*1.065;
        //fade_density   += stratocumuli_fade * 0.15;
        fade_density *= 1.5;

        powder_k.x      = 30.0 - sqrt(fade_top) * 15.5;
        powder_k.y      = 2.0 - sstep(altitude, minalt+depth*0.15, minalt+depth*0.33) * 1.0;

    float shape     = coverage;
    float slope     = sqrt(1.0 - saturate(shape));
    //float slope     = sqrt(1.0 - saturate(shape));

    if (slope > 1e-8) {  
    float d1        = 0.0;
    float n1        = 0.0;

    vec3 s1         = noise_3d(pos*0.45);

        n1         += (s1.x);          d1 += 1.0;
        n1         += s1.y*0.3;     d1 += 0.3;

        n1         /= d1;

        shape      -= (1.0-n1) * 0.35 * fade_worley * (1.0 + stratocumuli_fade * 0.25) * slope;

    if (shape < 0.0) return 0.0;

        slope     = sqrt(1.0 - saturate(shape));
    
    #if vcloud_shapeq>=1
    /*
        float n2        = 0.0;
        float d2        = 0.0;

        vec3 s2         = noise_3d(pos*1.0 + n1*0.2);

            n2         += s2.x;          d2 += 1.0;
            n2         += sqrt(s2.y)*0.5;     d2 += 0.5;

            n2         /= d2;

            shape      -= (1.0-n2)*0.18*fade_worley*(1.0 + stratocumuli_fade * 0.3) * slope;
    */
        pos.xy += shape * 0.45;
        shape -= value_3d(pos * 32.0) * 0.1 * slope;

    #endif
    
    #if (vcloud_shapeq>=2 && !(defined PD_REFLECTION))
        if (lod_dist < (vcloud_clip * 0.15)) {
            vec3 pos2       = pos + (n1)*0.12;
            float n3        = fbm(pos2*50.0, wind*0.2, 0.66, 2.6, 2);
                n3          = (1.0-n3)*0.07*(1.5-fade_worley);
            shape  -= mix(n3, 0.03, sstep(lod_dist, vcloud_clip * 0.12, vcloud_clip * 0.15)) * slope;
        } else {
            shape -= 0.03 * slope;
            //return 10.0;
        }
    #else
        shape  -= 0.03 * slope;
    #endif

        //shape   = float(shape < 0.01 && coverage > 0.01);
    }

        shape *= fade_density * vcloud_lowlayer_data.z;

    return max(shape, 0.0);
}

float vcloud_shape_midlayer(vec3 pos, float altitude) {
    vec3 wind       = vec3(vcloud_tick) * vcloud_midlayer_winddir;
    pos.xz         -= vec2(14.284, 24.4187) * pi * 715;
    pos            *= vcloud_midsize;
    pos            += wind*0.17;
    pos.x          *= 1.1;

    float fadenoise  = noise_2d(pos.xz * 0.002  - wind.xz * 0.00012).b - 0.2 + vcloud_midlayer_data.w;
        fadenoise   += vcloud_midlayer_override;
    float altostratus_fade = sqr(sstep(fadenoise, 0.45, 0.6));
        //altostratus_fade = 0.0;
    float cov_fade  = sstep(fadenoise, 0.25, 0.54) * (1.0 - sstep(fadenoise, 0.56, 0.75));
    if (cov_fade > 0.96) return 0.0;

    float minalt        = mix(vcloud_altocumulus_min, vcloud_altostratus_min, altostratus_fade);
    float maxalt        = mix(vcloud_altocumulus_max, vcloud_altostratus_max, altostratus_fade);
    float depth         = maxalt - minalt;

    if (altitude > maxalt || altitude < minalt) return 0.0;


    /* ------ coverage section ------ */

    float erode_low     = 1.0-sstep(altitude, minalt, minalt+depth*0.16);
    float erode_high    = sstep(altitude, minalt+depth*0.2, maxalt);
    float fade_low      = sstep(altitude, minalt, minalt+depth*0.1);
    float fade_high     = 1.0-sstep(altitude, minalt+depth*0.5, maxalt);

    if (max(fade_low, fade_high) < 0.01) return 0.0;

    float local_cov = noise_2d(pos.xz*0.0005).y;
        local_cov   = sstep(local_cov - altostratus_fade * 0.05 + vcloud_midlayer_data.y, 0.40, 0.70);

    if (local_cov>0.995) return 0.0;

    float nsize     = 0.035;

    if (altostratus_fade > 0.5) {
        nsize *= 0.5;
        pos.x  *= 1.4;
    }

    float coverage_bias = 0.46 - altostratus_fade * 0.12 + vcloud_midlayer_data.x;
        //coverage_bias += pow3(cov_fade) * 0.08;

    float coverage  = noise_2d(pos.xz*nsize).b;
        coverage    = (coverage - coverage_bias) * rcp(1.0 - saturate(coverage_bias));
        coverage   -= pow5(cov_fade) * 0.75;
        coverage   *= 1.0 - pow4(cov_fade);

        coverage   -= local_cov*0.5;
        coverage   *= (1.0-local_cov);

        coverage   *= fade_low;
        coverage   *= fade_high;
        coverage   -= erode_low*0.60;
        coverage   -= erode_high*1.0;

        coverage    = saturate(coverage * 1.5);

    if (coverage < 0.04) return 0.0;


    /* ------ shape detail section ------ */

    float fade_density  = 0.1 + sstep(altitude, minalt, minalt+depth*0.3)*0.2;
        fade_density   += sstep(altitude, minalt+depth*0.2, maxalt)*0.5 * (1.0 - altostratus_fade);

    float shape     = coverage;
    float slope     = sqrt(1.0 - saturate(shape));

    if (slope > 1e-8) {
        float d1        = 0.0;
        float n1        = 0.0;

        float s1size    = 0.5;

        if (altostratus_fade > 0.5) {
            s1size *= 0.45;
            pos    += coverage * 0.6;
        }

        vec3 s1         = noise_3d(pos*s1size);

            n1         += s1.x;          d1 += 1.0;
            n1         += s1.y*0.5;     d1 += 0.5;

            n1         /= d1;

            shape      -= (1.0-n1) * 0.45 * (1.0 - altostratus_fade * 0.3) * slope;

        if (shape < 0.0) return 0.0;

        #ifndef PD_REFLECTION
            vec3 pos2       = pos + vec3(n1, n1 * 0.1, n1 * 0.4)*0.4;
            if (altostratus_fade > 0.5) pos2 = pos * 0.45 + vec3(shape, 0.0, -shape) * 0.6;
            float n3        = value_3d(pos2*40.0 + wind*0.2);
                shape      -= n3*(0.15 + altostratus_fade * 0.02) * slope;
        #else
            shape  -= 0.03;
        #endif
    }

        //shape   = float(shape < 0.01 && coverage > 0.01);

        shape *= fade_density;

        shape *= 0.8 - altostratus_fade * 0.45;
        shape *= vcloud_midlayer_data.z;

    return max(shape, 0.0);
}

float vcloud_shape_highlayer(vec3 pos, float altitude) {
    vec3 wind       = vec3(vcloud_tick) * vcloud_highlayer_winddir;
    pos.xz         -= vec2(17.5284, 29.98487) * tau * 400;
    pos            *= vcloud_highsize;
    pos            += wind*0.2;

    float cirrocumuli_fade = vcloud_highlayer_data.w;
        cirrocumuli_fade  += vcloud_highlayer_override;
        cirrocumuli_fade   = saturate(cirrocumuli_fade);
    float cov_fade      = sstep(cirrocumuli_fade, 0.1, 0.49) - sstep(cirrocumuli_fade, 0.51, 0.9);
    if (cov_fade > 0.96) return 0.0;

    float minalt        = mix(vcloud_cirrus_min, vcloud_cirrocumuli_min, cirrocumuli_fade);
    float maxalt        = mix(vcloud_cirrus_max, vcloud_cirrocumuli_max, cirrocumuli_fade);
    float depth         = maxalt - minalt;

    if (altitude > maxalt || altitude < minalt) return 0.0;

    /* ------ coverage section ------ */

    float erode_low     = 1.0-sstep(altitude, minalt, minalt+depth*0.20);
    float erode_high    = sstep(altitude, minalt+depth*0.24, maxalt);
    float fade_low      = sstep(altitude, minalt, minalt+depth*0.10);
    float fade_high     = 1.0-sstep(altitude, minalt+depth*0.5, maxalt);

    if (cirrocumuli_fade < 0.5) pos.x *= 0.4;

    float local_cov = noise_2d(pos.xz*0.0005).x;
        local_cov   = sstep(local_cov + vcloud_highlayer_data.y, 0.37, 0.67 - cirrocumuli_fade * 0.1);

    if (local_cov>0.995) return 0.0;

    if (cirrocumuli_fade > 0.5) pos *= 3.0;

    float coverage  = noise_2d(pos.xz*0.04).b;
        coverage   -= 0.52 - cirrocumuli_fade * 0.08 - vcloud_highlayer_data.x;
        coverage   -= cov_fade * 0.55;

        coverage   -= local_cov*(0.5 + cirrocumuli_fade * 0.1);
        coverage   *= (1.0-local_cov);

        coverage   *= fade_low;
        coverage   *= fade_high;
        coverage   -= erode_low*(0.25);
        coverage   -= sqrt(erode_high)*(0.5);
        coverage    = max(coverage, 0.0);
    
    if (coverage < 0.01) return 0.0;


    /* ------ shape detail section ------ */

    float shape     = coverage;

    float fade_density  = 0.1 + sstep(altitude, minalt+depth*0.1, minalt+depth*0.4)*0.8;

    if (cirrocumuli_fade < 0.5) pos.z *= 0.3;
    else pos *= 0.45;

        vec3 pos1       = pos + shape * 0.5;
            pos1.x     += shape * 0.75;
        float n1        = fbm(pos1*10.0, wind*0.2, 0.66, 2.6, 2);
            shape      -= (1.0-n1)*0.13;

        if (shape < 0.0) return 0.0;

        #ifndef PD_REFLECTION
            vec3 pos2       = pos + shape * 0.5;
                pos2.x     += shape * 0.5;
            float n2        = value_3d(pos2*50.0 + wind*0.2);
                shape      -= n2*0.06;
        #else
            shape  -= 0.02;
        #endif

        shape  *= 0.18 + cirrocumuli_fade * 0.2;
        shape  *= vcloud_highlayer_data.z;

    return max(shape, 0.0);
}

float vcloud_shape_container(vec3 pos, out vec2 powder_k) {
    float altitude  = pos.y;
    float lod_dist  = distance(pos, cameraPosition);

    powder_k        = vec2(30.0, 2.0);

    //if (lod_dist > (vcloud_clip * 0.75)) return 10.0;

    if (altitude < vcloud_lowmax) {
        #ifdef vcloud_lowlayer
            return vcloud_shape_lowlayer(pos, altitude, lod_dist, powder_k);
        #else
            return 0.0;
        #endif
    } else if (altitude < vcloud_midmax) {
        #ifdef vcloud_midlayer
            powder_k.x  = 40.0;
            return vcloud_shape_midlayer(pos, altitude);
        #else
            return 0.0;
        #endif
    } else {
        #if vcloud_highlayer == 2
            powder_k.x  = 50.0;
            return vcloud_shape_highlayer(pos, altitude);
        #else
            return 0.0;
        #endif
    }
}
float vcloud_shape_container(vec3 pos) {
    vec2 dump;
    return vcloud_shape_container(pos, dump);
}

float pcloud_shape_container(vec3 pos, out float powder_k) {
    float altitude  = pos.y;
    float lod_dist  = distance(pos, cameraPosition);

    #if vcloud_highlayer == 1
        powder_k    = 30.0;
        return vcloud_shape_highlayer(pos, altitude);
    #else
        return 0.0;
    #endif
}
float pcloud_shape_container(vec3 pos) {
    float dump;
    return pcloud_shape_container(pos, dump);
}

/* ------ lighting functions ------ */


float vcloud_direct_od(in vec3 pos, const int steps) {
    //const float basestep = mix(150.0, 20.0, saturate(float(steps-1)/4.0));
    //const float exponent = mix(2.2, 1.1, saturate(float(steps-8)/8.0));
    const float basestep    = (22.2 * 6.0) / steps;
    const float exponent    = 2.2;

    float stepsize  = basestep;
    float prev_step = stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += cloudLightvector * stepsize) {
        if(pos.y > vcloud_maxalt || pos.y < vcloud_minalt) break;

        prev_step = stepsize;
        stepsize *= exponent;

        float density = vcloud_shape_container(pos);
        if (density <= 0.0) continue;
        od += density*prev_step;
    } 

    return od;
}
float vcloud_direct_od(in vec3 pos, const int steps, float noise) {
    //const float basestep = mix(150.0, 20.0, saturate(float(steps-1)/4.0));
    //const float exponent = mix(2.2, 1.1, saturate(float(steps-8)/8.0));
    const float basestep    = (22.2 * 6.0) / steps;
    const float exponent    = 2.2;

    float stepsize  = basestep;
    float prev_step = stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += cloudLightvector * stepsize) {
        if(pos.y > vcloud_maxalt || pos.y < vcloud_minalt) break;

        prev_step = stepsize;
        stepsize *= exponent;

        float density = vcloud_shape_container(pos + (noise * stepsize) * cloudLightvector);
        if (density <= 0.0) continue;
        od += density*prev_step;
    } 

    return od;
}

float vcloud_sky_od(in vec3 pos, const int steps) {
    const vec3 direction = vec3(0, 1, 0);

    float stepsize = vcloud_depth / steps;
        stepsize  *= 1.0 - linStep(pos.y, vcloud_minalt, vcloud_maxalt)*0.9;

    vec3 increment = direction * stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += increment) {
        if(pos.y > vcloud_maxalt) break;

        float density = vcloud_shape_container(pos);
        if (density <= 0.0) continue;
        od += density*stepsize;
    } 

    return od;
}
float vcloud_sky_od(in vec3 pos, const int steps, float noise) {
    const vec3 direction = vec3(0, 1, 0);

    float stepsize = vcloud_depth / steps;
        stepsize  *= 1.0 - linStep(pos.y, vcloud_minalt, vcloud_maxalt)*0.9;

    vec3 increment = direction * stepsize;

        pos       += direction * noise * stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += increment) {
        if(pos.y > vcloud_maxalt) break;

        float density = vcloud_shape_container(pos);
        if (density <= 0.0) continue;
        od += density*stepsize;
    } 

    return od;
}

float vcloud_bouncelight(vec3 pos) {
    const float high    = vcloud_minalt+vcloud_lowdepth*0.3;
    return pow3(1.0-sstep(pos.y, vcloud_minalt, high));
}

float pcloud_direct_od(in vec3 pos, const int steps) {
    //const float basestep = mix(150.0, 20.0, saturate(float(steps-1)/4.0));
    //const float exponent = mix(2.2, 1.1, saturate(float(steps-8)/8.0));
    const float basestep    = 22.2;
    const float exponent    = 2.2;

    float stepsize  = basestep;
    float prev_step = stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += cloudLightvector * stepsize) {
        if(pos.y > vcloud_maxalt || pos.y < vcloud_minalt) break;

        prev_step = stepsize;
        stepsize *= exponent;

        float density = pcloud_shape_container(pos);
        if (density <= 0.0) continue;
        od += density*prev_step;
    } 

    return od;
}

float pcloud_sky_od(in vec3 pos, const int steps) {
    const vec3 direction = vec3(0, 1, 0);

    float stepsize = vcloud_depth / steps;
        stepsize  *= 1.0 - linStep(pos.y, vcloud_minalt, vcloud_maxalt)*0.9;

    vec3 increment = direction * stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += increment) {
        if(pos.y > vcloud_maxalt) break;

        float density = pcloud_shape_container(pos);
        if (density <= 0.0) continue;
        od += density*stepsize;
    } 

    return od;
}