
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Lee, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

/*
====================================================================================================

        License Terms for Academy Color Encoding System Components

        Academy Color Encoding System (ACES) software and tools are provided by the
        Academy under the following terms and conditions: A worldwide, royalty-free,
        non-exclusive right to copy, modify, create derivatives, and use, in source and
        binary forms, is hereby granted, subject to acceptance of this license.

        Copyright © 2015 Academy of Motion Picture Arts and Sciences (A.M.P.A.S.).
        Portions contributed by others as indicated. All rights reserved.

        Performance of any of the aforementioned acts indicates acceptance to be bound
        by the following terms and conditions:

        * Copies of source code, in whole or in part, must retain the above copyright
        notice, this list of conditions and the Disclaimer of Warranty.

        * Use in binary form must retain the above copyright notice, this list of
        conditions and the Disclaimer of Warranty in the documentation and/or other
        materials provided with the distribution.

        * Nothing in this license shall be deemed to grant any rights to trademarks,
        copyrights, patents, trade secrets or any other intellectual property of
        A.M.P.A.S. or any contributors, except as expressly stated herein.

        * Neither the name "A.M.P.A.S." nor the name of any other contributors to this
        software may be used to endorse or promote products derivative of or based on
        this software without express prior written permission of A.M.P.A.S. or the
        contributors, as appropriate.

        This license shall be construed pursuant to the laws of the State of
        California, and any disputes related thereto shall be subject to the
        jurisdiction of the courts therein.

        Disclaimer of Warranty: THIS SOFTWARE IS PROVIDED BY A.M.P.A.S. AND CONTRIBUTORS
        "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
        THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
        NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL A.M.P.A.S., OR ANY
        CONTRIBUTORS OR DISTRIBUTORS, BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
        SPECIAL, EXEMPLARY, RESITUTIONARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
        LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
        PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
        LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
        OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
        ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

        WITHOUT LIMITING THE GENERALITY OF THE FOREGOING, THE ACADEMY SPECIFICALLY
        DISCLAIMS ANY REPRESENTATIONS OR WARRANTIES WHATSOEVER RELATED TO PATENT OR
        OTHER INTELLECTUAL PROPERTY RIGHTS IN THE ACADEMY COLOR ENCODING SYSTEM, OR
        APPLICATIONS THEREOF, HELD BY PARTIES OTHER THAN A.M.P.A.S.,WHETHER DISCLOSED OR
        UNDISCLOSED.

====================================================================================================
*/

#ifndef incACESMT
    #include "transforms.glsl"
#endif

#include "functions.glsl"
#include "spline.glsl"

/*
this is the accurate academy color encoding reference rendering transform that is given by A.M.P.A.S.
*/

const float dim_surround_gamma = 0.911; //default 0.9811

const float rrt_glow_gain   = 0.28;     //default 0.05
const float rrt_glow_mid    = 0.08;     //default 0.08

const float rrt_red_scale   = 1.0;     //default 0.82
const float rrt_red_privot  = 0.03;     //default 0.03
const float rrt_red_hue     = 0.0;      //default 0.0
const float rrt_red_width   = 135.0;    //default 135.0

const float rrt_sat_factor  = 0.92;     //default 0.96
const float odt_sat_factor = 0.93;      //default 0.93

vec3 darkToDimSurround(vec3 linear_cv) {
    vec3 xyz    = linear_cv * AP1_XYZ;
    vec3 xyY    = XYZ_xyY(xyz);
        xyY.z   = clamp(xyY.z, 0.0, 65535.0);
        xyY.z   = pow(xyY.z, dim_surround_gamma);
        xyz     = xyY_XYZ(xyY);

    return xyz * XYZ_AP1;
}

mat3x3 calcSaturationMatrix(const float sat, const vec3 rgb_y) {
    mat3x3 M;
        M[0].r = (1.0 - sat) * rgb_y.r + sat;
        M[1].r = (1.0 - sat) * rgb_y.r;
        M[2].r = (1.0 - sat) * rgb_y.r;

        M[0].g = (1.0 - sat) * rgb_y.g;
        M[1].g = (1.0 - sat) * rgb_y.g + sat;
        M[2].g = (1.0 - sat) * rgb_y.g;

        M[0].b = (1.0 - sat) * rgb_y.b;
        M[1].b = (1.0 - sat) * rgb_y.b;
        M[2].b = (1.0 - sat) * rgb_y.b + sat;

    return M;
}

//mat3x3 rrt_sat_matrix   = calcSaturationMatrix(rrt_sat_factor, AP1_RGB_Y);  //seems to be redundant to using dot() for saturation

vec3 rrtSweeteners(vec3 aces) {
    /* glow module */
    float saturation    = rgb_saturation(aces);
    float yc_in         = rgb_yc(aces);
    float s             = sigmoid_sharper((saturation - 0.4) * rcp(0.2));
    float added_glow    = 1.0 + glow_fwd(yc_in, rrt_glow_gain * s, rrt_glow_mid);
    
        aces   *= added_glow;

    /* red modifier module */
    float hue           = rgb_hue(aces);
    float centered_hue  = center_hue(hue, rrt_red_hue);
    float hue_weight    = cubic_basis_shaper(centered_hue, rrt_red_width);

        aces.r += hue_weight * saturation * (rrt_red_privot - aces.r) * (1.0 - rrt_red_scale);

    /* aces to rgb */
        aces    = clamp(aces, 0.0, 65535.0);

        vec3 rgb_pre    = aces * AP0_AP1;
            rgb_pre     = clamp(rgb_pre, 0.0, 65535.0);

    /* global desaturation */
        rgb_pre         = mix(vec3(dot(rgb_pre, AP1_RGB_Y)), rgb_pre, rrt_sat_factor);
        //rgb_pre         = rgb_pre * rrt_sat_matrix;
    
    return rgb_pre;
}

/* takes input as academy color and returns oces */
vec3 academyRRT(vec3 aces) {
    vec3 rgb_pre        = rrtSweeteners(aces);

    /* apply tonescale in rgb space */
    vec3 rgb_post;
        rgb_post.r      = segmented_spline_c5_fwd(rgb_pre.r);
        rgb_post.g      = segmented_spline_c5_fwd(rgb_pre.g);
        rgb_post.b      = segmented_spline_c5_fwd(rgb_pre.b);

    /* rgb space to oces */
    return rgb_post * AP1_AP0;
}

vec3 odtSRGB_D65(vec3 oces) {
    vec3 rgb_pre = oces * AP0_AP1;

    segmented_spline_param_c9 odt_48nit = segmented_spline_param_c9(
        float[10](-1.6989700043, -1.6989700043, -1.4779000000, -1.2291000000, -0.8648000000, -0.4480000000, 0.0051800000, 0.4511080334, 0.9113744414, 0.9113744414),
        float[10](0.5154386965, 0.8470437783, 1.1358000000, 1.3802000000, 1.5197000000, 1.5985000000, 1.6467000000, 1.6746091357, 1.6878733390, 1.6878733390),
        vec2(segmented_spline_c5_fwd(0.18 * exp2(-6.5)), 0.02),
        vec2(segmented_spline_c5_fwd(0.18), 4.8),
        vec2(segmented_spline_c5_fwd(0.18 * exp2(6.5)), 48.0),
        0.0,
        0.04
    );

    /* apply tonescale in rgb space */
    vec3 rgb_post;
        rgb_post.r      = segmented_spline_c9_fwd(rgb_pre.r, odt_48nit);
        rgb_post.g      = segmented_spline_c9_fwd(rgb_pre.g, odt_48nit);
        rgb_post.b      = segmented_spline_c9_fwd(rgb_pre.b, odt_48nit);

    /* black and white points for cinema system */
    const float cinema_white    = 48.0;
    const float cinema_black    = 0.0;     //white / 2400.0, default 0.02

    /* scale luminance to linear */
    vec3 linear_cv;
        linear_cv.r     = y_linVC(rgb_post.r, cinema_white, cinema_black);
        linear_cv.g     = y_linVC(rgb_post.g, cinema_white, cinema_black);
        linear_cv.b     = y_linVC(rgb_post.b, cinema_white, cinema_black);

        linear_cv       = darkToDimSurround(linear_cv);

    /* global desaturation */
        linear_cv       = mix(vec3(dot(linear_cv, AP1_RGB_Y)), linear_cv, odt_sat_factor);

    /* convert to display primary encoding */
    vec3 xyz    = linear_cv * AP1_XYZ;
        xyz     = xyz * D60_D65;

        linear_cv   = xyz * XYZ_sRGB;

    return saturate(linear_cv);
}

/*
this is an approximated verison of the reference rendering transform in order to reduce the performance cost
*/

struct academy_color_params {
    float slope;
    float toe;
    float shoulder;
    float black_clip;
    float white_clip;
};

vec3 academySplineFit(vec3 rgb_pre, const academy_color_params curve) {
    /* approximated rrt spline based on unreal engine */
    #ifdef MC_GL_RENDERER_INTEL
        float toe_scale   = 1.0 + curve.black_clip - curve.toe;
        float shoulder_scale = 1.0 + curve.white_clip - curve.shoulder;
    #else
        const float toe_scale   = 1.0 + curve.black_clip - curve.toe;
        const float shoulder_scale = 1.0 + curve.white_clip - curve.shoulder;
    #endif

    const float in_match    = 0.18;
    const float out_match   = 0.18;

    float toe_match = 0.0;

    if (curve.toe > 0.8) {
        toe_match   = (1.0 - curve.toe - out_match) / curve.slope + log10(in_match);
    } else {
        #ifdef MC_GL_RENDERER_INTEL
            float bt  = (out_match + curve.black_clip) / toe_scale - 1.0;
        #else
            const float bt  = (out_match + curve.black_clip) / toe_scale - 1.0;
        #endif
        
        toe_match   = log10(in_match) - 0.5 * log((1.0 + bt) * rcp(1.0 - bt)) * (toe_scale * rcp(curve.slope));
    }

    float straight_match    = (1.0 - curve.toe) / curve.slope - toe_match;
    float shoulder_match    = curve.shoulder / curve.slope - straight_match;

    vec3 log_color          = log10(rgb_pre);
    vec3 straight_color     = curve.slope * (log_color + straight_match);

    vec3 toe_color          = (-curve.black_clip) + (2.0 * toe_scale) * rcp(1.0 + exp((-2.0 * curve.slope / toe_scale) * (log_color - toe_match)));
    vec3 shoulder_color     = (1.0 + curve.white_clip) - (2.0 * shoulder_scale) * rcp(1.0 + exp((2.0 * curve.slope / shoulder_scale) * (log_color - shoulder_match)));

        toe_color           = smallerThanVec3(log_color, vec3(toe_match), toe_color, straight_color);
        shoulder_color      = greaterThanVec3(log_color, vec3(shoulder_match), shoulder_color, straight_color);

    vec3 t      = saturate((log_color - toe_match) * rcp(shoulder_match - toe_match));
        t       = shoulder_match < toe_match ? 1.0 - t : t;
        t       = (3.0 - 2.0 * t) * t * t;

    return mix(toe_color, shoulder_color, t);
}

vec3 academyApprox(vec3 aces) {
    const academy_color_params curve = academy_color_params(
        0.91,
        0.51,
        0.23,
        0.0,
        0.035
    );

    vec3 rgb_pre        = rrtSweeteners(aces);

    vec3 mapped_color   = academySplineFit(rgb_pre, curve);

    /* global desaturation */
        mapped_color        = mix(vec3(dot(mapped_color, AP1_RGB_Y)), mapped_color, odt_sat_factor);

    return max(mapped_color, 0.0) * AP1_AP0;
}


/*
"Packed" ACES transforms
*/
vec3 ACES_LINEAR_SRGB_RRT(vec3 LIN) {   //linear sRGB in
        LIN    *= 1.31;
    vec3 ACES   = LIN * sRGB_AP0;
        ACES    = academyRRT(ACES);
        ACES    = odtSRGB_D65(ACES);

    return LinearToSRGB(ACES);
}
vec3 ACES_LINEAR_SRGB(vec3 LIN) {
    vec3 ACES   = LIN * sRGB_AP0;
        ACES    = academyApprox(ACES);
        ACES    = ACES * AP0_sRGB;

    return LinearToSRGB(ACES);
}

vec3 ACES_AP1_SRGB_RRT(vec3 AP1) {      //AP1/ACEScg in
        AP1    *= 1.313;
    vec3 ACES   = AP1 * AP1_AP0;
    ACES        = academyRRT(ACES);
    ACES        = odtSRGB_D65(ACES);

    return LinearToSRGB(ACES);
}
vec3 ACES_AP1_SRGB(vec3 AP1) {
    vec3 ACES   = AP1 * AP1_AP0;
        ACES    = academyApprox(ACES);
        ACES    = ACES * AP0_sRGB;

    return LinearToSRGB(ACES);
}


/*
this is a highly simplified version of the academy color encoding system rrt and odt
*/

/* sRGB_XYZ -> D65_D60 -> AP1 -> RRT */
const mat3 sRGB_AP1_RRT = mat3(
    0.59719, 0.35458, 0.04823,
    0.07600, 0.90834, 0.01566,
    0.02840, 0.13383, 0.83777
);
/* ODT -> XYZ -> D60_D65 -> XYZ_sRGB */
const mat3 ODT_sRGB = mat3(
     1.60475, -0.53108, -0.07367,
    -0.10208,  1.10813, -0.00605,
    -0.00327, -0.07276,  1.07602
);

vec3 academy_simplified(vec3 linear){
    const academy_color_params curve = academy_color_params(
        0.915,
        0.435,
        0.21,
        0.0,
        0.036
    );

	vec3 aces = linear*sRGB_AP1_RRT;

        aces  = academySplineFit(aces, curve);
        aces  = aces*ODT_sRGB;

    return LinearToSRGB(aces);
}