/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

#ifndef PD_FILTERLOD
    #define PD_FILTERLOD 1.0
#endif

const float kernelW_3x3[9]  = float[9](
    0.0625, 0.125, 0.0625,
    0.125,  0.250, 0.125,
    0.0625, 0.125, 0.0625
);

const ivec2 kernelO_3x3[9]  = ivec2[9](
    ivec2(-1, -1), ivec2(0, -1), ivec2(1, -1),
    ivec2(-1, 0),  ivec2(0, 0),  ivec2(1, 0),
    ivec2(-1, 1),  ivec2(0, 1),  ivec2(1, 1)
);

vec4 filter_3x3(sampler2D tex, sampler2D depth, vec2 coord, const int size) {
    ivec2 d_pos = ivec2(coord * viewSize * PD_FILTERLOD);
    ivec2 c_pos  = ivec2(coord * viewSize);

    vec4 center_col     = texelFetch(tex, c_pos, 0);

    float center_depth  = texelFetch(depth, d_pos, 0).x;
        center_depth    = depth_lin(center_depth) * far;

    vec4 filter_data    = texelFetch(colortex6, c_pos, 0);

    vec3 center_normal  = filter_data.rgb * 2.0 - 1.0;

    vec4 sumcol     = vec4(0.0);
    float sumweight = 0.0;

    for (int i = 0; i<9; i++) {
        ivec2 delta_pos  = ivec2(kernelO_3x3[i] * size);

        float blur  = kernelW_3x3[i];

        ivec2 depth_pos  = d_pos + ivec2(delta_pos * PD_FILTERLOD);
        float curr_depth = depth_lin(texelFetch(depth, depth_pos, 0).x) * far;
        float depth_weight = exp(-max(distance(curr_depth, center_depth) - 0.05, 0.0) * 1.5);

        ivec2 curr_pos   = c_pos + delta_pos;
        vec4 curr_data   = texelFetch(colortex6, curr_pos, 0);
        vec3 curr_normal = curr_data.rgb * 2.0 - 1.0;
        float normal_weight = max(dot(curr_normal, center_normal), 0.0);

        vec4 curr_col   = texelFetch(tex, curr_pos, 0);
        float curr_luma = get_luma(curr_col.rgb);

        float weight    = (normal_weight) * (depth_weight) * blur;

        sumcol     += curr_col * weight;
        sumweight  += weight;
    }

    sumweight   = max(sumweight, 1e-16);

    sumcol /= sumweight;

    return sumcol;
}