/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



#include "/lib/lighting/offset.glsl"

float get_vps_sigma(sampler2D tex, vec3 pos, float cd, float warp) {
    const float search_area = 25e-3;
    const float max_sigma   = 6e-4;
    const float light_size  = 20e-5;
    const int iterations    = shadow_vps_iterations;

    float zpos  = (-cd) * 0.5 + 0.5;
    //float dist  = 0.0;
    float sigma = 0.0;
    float depth = 0.0;
    int blockers = 0;

    float area  = (search_area / zpos) * rcp(warp);

    const int it  = iterations * iterations;
    const float w = 1.0 / it;

    float sdepth    = texture(tex, pos.xy).x;
    float s         = get_shadow(sdepth, cd);

    for (int i = -iterations; i<iterations; i++) {
        for (int j = -iterations; j<iterations; j++) {
            vec2 search_pos = pos.xy + (vec2(i, j) * area / (iterations));
            float d         = texture(tex, search_pos).x;

            //if ((s < 0.2) && ((cd - sdepth) < (cd-d))) continue;
            //if (d > sdepth) continue;

                sigma      += max(cd - d * 256.0 - 0.035, 0.0);
                //dist       += depth;
                blockers   += 1;
        }
    }
    sigma /= blockers;

    return clamp(sigma * light_size /*- 5e-5*/, /*- 1e-4*/ 0.0, max_sigma) * rcp(warp);
}

vec4 get_shadow30(vec2 pos, float cd, float sigma) {
    float ts    = 0.0;
    vec3 tc     = vec3(0.0);

    float noise = dither_bluenoise()*pi;
    vec2 n      = vec2(cos(noise), sin(noise));

    if (abs(sigma) < 8e-5) {
        /* optimize very small sigmas */
        float s0        = get_shadowtex_cubic(shadowtex0, pos.xy);
        float s1        = get_shadowtex_cubic(shadowtex1, pos.xy);

        bool translucent = distance(s1, s0)>0.02/256.0;

        ts += get_shadow(s1, cd);

        if (translucent) {
            tc += get_shadowcol(shadowcolor0, pos.xy);
        } else {
            tc += vec3(1.0);
        }

    } else if (sigma >= 0.0) {
        /* regular blur for positive sigma */
        const float w = 1.0 / 20.0;
        for (int i = 0; i<20; i++) {
            vec2 delta  = (poisson60o[i * 3] * n) * sigma;
            vec2 p  = pos + delta;

            float s0        = get_shadowtex(shadowtex0, p.xy);
            float s1        = get_shadowtex(shadowtex1, p.xy);

            bool translucent = distance(s1, s0)>0.02/256.0;

            ts += get_shadow(s1, cd);

            if (translucent) {
                tc += get_shadowcol(shadowcolor0, p.xy);
            } else {
                tc += vec3(1.0);
            }
        }
        ts *= w;
        tc *= w;

    } else {
        /* sharpening for negative sigma */
        const float w = 1.0 / 15.0;
        float weight    = w;
        for (int i = 0; i<15; i++) {
            vec2 delta  = (poisson60o[i * 4] * n) * sigma;
            vec2 p  = pos + delta;

            float s0        = get_shadowtex_cubic(shadowtex0, p.xy);
            float s1        = get_shadowtex_cubic(shadowtex1, p.xy);

            bool translucent = distance(s1, s0)>0.02/256.0;

            if (sigma < 0.0) {
                weight      = clamp((length(delta) - (0.25 * abs(sigma))) / abs(sigma), -w * 2.0, w * 2.0);
            }

            ts += get_shadow(s1, cd) * weight;

            if (translucent) {
                tc += get_shadowcol(shadowcolor0, p.xy);
            } else {
                tc += vec3(1.0);
            }
        }
        tc *= w;
    }
    

    return vec4(tc, saturate(ts));
}