/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/


vec3 shadow_scenespace(vec3 pos) {
        pos    = pos * 2.0 - 1.0;
        pos     = projMAD3(shadowProjectionInverse, pos);
        pos     = transMAD(shadowModelViewInverse, pos);

    return pos;
}

float dither_bluenoise_gi() {
    ivec2 coord = ivec2(fract(gl_FragCoord.xy/256.0)*256.0);
    float noise = texelFetch(noisetex, coord, 0).a;

        noise   = fract(noise+float(frameCounter)/(8.0));

    return noise;
}

vec4 get_shadowtex_texel(sampler2D tex, vec2 pos) {
    ivec2 coord     = ivec2(pos * shadowMapResolution);

    return texelFetch(tex, coord, 0);
}

vec2 fermats_spiral_goldenS(float i, float total) {
	float theta = i * golden_angle;
	return vec2(sin(theta), cos(theta)) * sqr(i / total);
}
vec2 circleMap(in float index, in float count) {
    float goldenAngle = tau / ((sqrt(5.0) * 0.5 + 0.5) + 1.0);
    return vec2(cos(index * goldenAngle), sin(index * goldenAngle)) * sqrt(index / count);
}

vec3 compute_rsm(vec3 scenenormal, vec3 shadowspace_position, vec3 scenepos, float cd, float skylight) {
    const int steps   = rsm_steps;

    float bluenoise     = dither_bluenoise_gi();

    const float dither_scale = 32.0;
    float dither    = dither_scale * bluenoise;

    vec3 shadownormal = normalize(mat3(shadowModelView) * scenenormal);

    vec3 illum  = vec3(0.0);
    float weight = 1.0;

    float radius    = rsm_size * rcp(2048.0);

    for (int i = 0; i < steps; i++) {
        //vec2 offset     = fermats_spiral_goldenS(i * dither_scale + dither, steps * dither_scale) * radius;
        vec2 offset     = circleMap(float(i) * dither_scale + dither, steps * dither_scale) * radius;

        if (dot(shadownormal.xy, offset) <= 0.0) offset = -offset;

        vec3 offset_pos     = shadowspace_position + vec3(offset, 0.0);
        vec3 pos_warped     = offset_pos;
            pos_warped.xy   = warp_shadowmap(pos_warped.xy * 2.0 - 1.0) * 0.5 + 0.5;

        float shadowdepth   = get_shadowtex_texel(shadowtex1, pos_warped.xy).x;

        if ((shadowdepth * 256.0 + 0.02) < cd) {
            weight++;
            continue;
        }

        vec3 samplepos      = vec3(offset_pos.xy, shadowdepth);
        float samplelength  = length(samplepos - shadowspace_position) * shadowMapResolution * rcp(pi);

        if (samplelength > rsm_size) continue;
        if (samplelength < 1e-4) continue;

        vec3 samplepos_norm     = normalize(samplepos - shadowspace_position) * vec3(1.0, 1.0, -1.0);

        weight++;

        if (samplelength > rsm_size) continue;

        float falloff   = rcp(2e-2 + sqr(samplelength * rcp(sqr(pi))));
            falloff    *= (1.0 - sstep(samplelength, rsm_size*0.8, rsm_size));
            //falloff   = 1.0 - sstep(samplelength, 0.999, 1.001);

        if (falloff < 1e-8) continue;

        float diffuse   = saturate(dot(shadownormal, samplepos_norm));

        if (diffuse <= 0.0) continue;

        vec3 normal     = texture(shadowcolor1, pos_warped.xy).xyz * 2.0 - 1.0;
            normal      = -normal;

        float diff_bounce = saturate(dot(normal, samplepos_norm));

        if (diff_bounce <= 0.0) continue;

        vec4 x          = get_shadowtex_texel(shadowcolor0, pos_warped.xy);

        //if (x.a < 1.0 && x.a > 0.9) x.rgb = vec3(0.35, 0.74, 1.0) * 0.75;

        float shadow_skylight = x.a * 2.0 - 1.0;
        float bleeding_mask = shadow_skylight - skylight;
            bleeding_mask   = pow6(bleeding_mask);
            bleeding_mask   = saturate(0.05 * rcp(max(bleeding_mask, 0.05)));

        vec3 lsample    = SRGBtoACEScg(x.rgb);

        if (lsample != lsample) lsample = vec3(0.0);

        illum  += lsample * falloff * diffuse * diff_bounce * bleeding_mask;
    }

    return illum * rcp(weight);
}
