
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



#include "/lib/universal.glsl"

/* bloom downsampling method based on chocapic13's shaders */

/* ------ framebuffer attachment outputs ------ */
/*DRAWBUFFERS:4*/
layout(location = 0) out vec4 blur_data;

/* ------ inputs from vertex stage ------ */
in vec2 coord;

/* ------ uniforms ------ */
uniform vec2 pixelSize;
uniform vec2 viewSize;

#if pass == 0
    uniform sampler2D colortex0;
#elif pass == 1
    uniform sampler2D colortex4;
#endif


void main() {
    #if pass == 0
        vec2 rscale    = max(viewSize, vec2(1920.0, 1080.0))/vec2(1920.0, 1080.0);
        vec2 qrescoord = (gl_FragCoord.xy*2.0*pixelSize-vec2(0.0, 0.5))*rscale;
    #elif pass == 1
        vec2 qrescoord = gl_FragCoord.xy*2.0*pixelSize+vec2(0.0, 0.25);
    #endif


    #if pass == 0
        #define coltex colortex0
    #elif pass == 1
        #define coltex colortex4
    #endif

    //0.5
    blur_data   = texture(coltex, qrescoord-1.0*vec2(pixelSize.x, pixelSize.y))/4.0*0.5;
    blur_data  += texture(coltex, qrescoord+1.0*vec2(pixelSize.x, pixelSize.y))/4.0*0.5;
    blur_data  += texture(coltex, qrescoord+1.0*vec2(-pixelSize.x, pixelSize.y))/4.0*0.5;
    blur_data  += texture(coltex, qrescoord+1.0*vec2(pixelSize.x, -pixelSize.y))/4.0*0.5;

    //0.25
    blur_data  += texture(coltex, qrescoord-2.0*vec2(pixelSize.x, 0.0))/2.0*0.125;
    blur_data  += texture(coltex, qrescoord+2.0*vec2(0.0, pixelSize.y))/2.0*0.125;
    blur_data  += texture(coltex, qrescoord+2.0*vec2(-pixelSize.x, 0.0))/2.0*0.125;
    blur_data  += texture(coltex, qrescoord+2.0*vec2(0.0, -pixelSize.y))/2.0*0.125;

    //0.125
    blur_data  += texture(coltex, qrescoord-2.0*vec2(pixelSize.x, pixelSize.y))/4.0*0.125;
    blur_data  += texture(coltex, qrescoord+2.0*vec2(pixelSize.x, pixelSize.y))/4.0*0.125;
    blur_data  += texture(coltex, qrescoord+2.0*vec2(-pixelSize.x, pixelSize.y))/4.0*0.125;
    blur_data  += texture(coltex, qrescoord+2.0*vec2(pixelSize.x, -pixelSize.y))/4.0*0.125;

    blur_data  += texture(coltex, qrescoord)*0.125;

    #undef coltex


    #if pass == 0
        if (qrescoord.x>1.0-3.5*pixelSize.x || qrescoord.y>1.0-3.5*pixelSize.y || qrescoord.x<3.5*pixelSize.x || qrescoord.y<3.5*pixelSize.y) blur_data = vec4(0.0);
    #endif
    
    blur_data.a = 1.0;
    blur_data   = clamp_drawbuffer(blur_data);
}