/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/


#include "/lib/universal.glsl"

/*
const bool colortex0MipmapEnabled = true;
*/

/* ------ framebuffer attachment outputs ------ */
/*DRAWBUFFERS:0*/
layout(location = 0) out vec4 scenecol_data;

/* ------ inputs from vertex stage ------ */
in vec2 coord;

/* ------ uniforms ------ */
uniform float aspectRatio;
uniform float centerDepthSmooth;

uniform vec2 viewSize;

uniform mat4 gbufferModelViewInverse, gbufferProjectionInverse;
uniform mat4 gbufferModelView, gbufferProjection;

uniform sampler2D colortex0;
uniform sampler2D colortex3;
uniform sampler2D depthtex1;

float screenspaceToViewspace(float depth, mat4 projectionInverse) {
	depth = depth * 2.0 - 1.0;
	return projectionInverse[3].z / (projectionInverse[2].w * depth + projectionInverse[3].w);
}

const float sensorWidth = cameraSensorWidth * 1e-3;

vec2 getDistOffset(vec2 prep, float angle, vec2 pxoffset) {
    return pxoffset * angle + prep * dot(prep, pxoffset) * (1.0 - angle);
}

float calculate_CoC(float dist, float focus, float focalLength, float aperture) {
    float n     = aperture * (focalLength * (dist - focus));
    float d     = dist * (focus - focalLength);

    return abs(n) * rcp(max(d, 1e-20)) * rcp(sensorWidth) * 1e3;
}

vec3 depthOfField(float depth, bool hand) {
    //Bit faster than the DOF in NV, and I have some plans and ideas to make it even faster while looking better.

    float focus = abs(screenspaceToViewspace(hand ? centerDepthSmooth + 0.45 : centerDepthSmooth, gbufferProjectionInverse));
    float dist = abs(screenspaceToViewspace(depth, gbufferProjectionInverse));
    
    float focalLength   = 0.5 * sensorWidth * gbufferProjection[0].x;
    float aperture      = focalLength * rcp(cameraFStop);
    float CoC           = calculate_CoC(dist, focus, aperture, focalLength);
    
    vec3 result     = vec3(0.0);

    const float samples = dofSamples;

    vec2 bokehAngle     = vec2(0.0, 1.0);
    
    float sizeCorrection = rcp(sqrt(samples) * 1.35914091423);
    vec2 offsetScale    = (sizeCorrection * aperture) * vec2(1.0, aspectRatio);

    float lod_k = 0.002 * viewSize.y;
    //float lod = log2(lod_k * CoC) * rcp(samples);

    float r = 1.0;
    float rotation  = 2.4;
    mat2 rot = mat2(cos(rotation), -sin(rotation), sin(rotation), cos(rotation));
    vec3 weight;

    vec2 tocenter = coord - 0.5;
    vec2 prep = normalize(vec2(tocenter.y, -tocenter.x));
    float angle = cos(length(tocenter.xy) * 2.221 * 2.0 * cameraBarrelDistortion);


    for(int i = 0; i < dofSamples; ++i) {
        r += 1.0 / r;
        bokehAngle = rot * bokehAngle;

        vec2 rAngle = (r - 1.0) * bokehAngle;

        vec2 offset = getDistOffset(prep, 1.0, rAngle) * sizeCorrection * 0.5 + 0.5;
        vec3 bokeh  = texture(colortex3, offset / pow(2.0, LensLOD) + bokehOffset).rgb;

        if(bokeh.r > 0.0 || bokeh.g > 0.0 || bokeh.b > 0.0) {
            vec2 offset2    = getDistOffset(prep, angle, rAngle * CoC) * offsetScale;

            float ndist = abs(screenspaceToViewspace(texture(depthtex1, coord + offset2).r, gbufferProjectionInverse));
            float nCoC  = calculate_CoC(ndist, focus, aperture, focalLength);

            offset  = getDistOffset(prep, angle, rAngle * nCoC) * offsetScale;

            #ifdef dofLodEnabled
            float lod   = clamp(log2(lod_k * nCoC) * rcp(samples) * 60, 0.0, 4.0);
            #else
            const int lod = 0;
            #endif

            result += textureLod(colortex0, coord + offset, lod).rgb * bokeh;
            weight += bokeh;
        }
    }

    return result / weight;
}

void main() {
    float depth1 = texture(depthtex1, coord).r;
    bool hand = depth1 < 0.7;

    #ifdef dof_exclude_hand
        if(hand) discard;
    #endif

    scenecol_data = stexLod(colortex0, 0);

    #ifdef dof_enabled 
        // if not defined, the whole program gets disabled (but the option doesn't appear if the macro is unused, reee)
        scenecol_data.rgb = depthOfField(depth1, hand);
    #endif

    scenecol_data = make_drawbuffer(scenecol_data);
}