#version 450 compatibility

/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



#include "/lib/universal.glsl"

/*
const int colortex0Format = RGBA16F;
const int colortex1Format = RGBA16;
const int colortex2Format = RGBA16;
const int colortex3Format = RGBA16F;
const int colortex4Format = RGBA16F;
const int colortex5Format = RGBA16F;
const int colortex6Format = RGBA8;
const int colortex7Format = RGBA16F;

const int noiseTextureResolution = 256;

const vec4 colortex0ClearColor = vec4(0.0, 0.0, 0.0, 1.0);
const vec4 colortex3ClearColor = vec4(0.0, 0.0, 0.0, 0.0);

const bool colortex5Clear = false;
*/


/* ------ framebuffer attachment outputs ------ */
/*DRAWBUFFERS:3*/
layout(location = 0) out vec4 indirectLight_data;


/* ------ inputs from vertex stage ------ */
in vec2 coord;


/* ------ uniforms ------ */
uniform int frameCounter;

uniform float eyeAltitude;
uniform float far, near;

uniform vec2 pixelSize, viewSize;
uniform vec2 taaOffset;

uniform mat4 gbufferModelViewInverse, gbufferModelView;
uniform mat4 gbufferProjectionInverse, gbufferProjection;

uniform sampler2D colortex1;

uniform sampler2D depthtex0;

uniform sampler2D noisetex;
uniform sampler3D depthtex2;


/* ------ includes ------ */
#include "/lib/util/encoders.glsl"
#include "/lib/atmosphere/constants.glsl"
#include "/lib/fragment/dither.glsl"
#include "/lib/fragment/lowresCommon.glsl"
#include "/lib/util/transforms.glsl"
#include "/lib/lighting/ao.glsl"

void main() {
    float bluenoise    = dither_bluenoise();

    /* ------- ambient occlusion just works ------ */
    indirectLight_data = vec4(0.0, 0.0, 0.0, 1.0);

    #ifdef ambient_occlusion_enabled
        vec2 aoCoord   = coord * 2.0;
        
        if (clamp(aoCoord, 0.0 - pixelSize, 1.0 + pixelSize) == aoCoord) {
            float depth_3x3          = max_depth3x3(depthtex0, aoCoord, pixelSize * sqrt(2.0));

            if (landMask(depth_3x3)) {
                vec2 coord           = saturate(aoCoord);
                float sceneDepth     = texelFetch(depthtex0, ivec2(coord * viewSize), 0).x;

                vec4 tex1            = texelFetch(colortex1, ivec2(coord * viewSize), 0);

                vec3 viewPos         = screenSpaceToViewSpace(vec3(coord, sceneDepth), gbufferProjectionInverse);
                vec3 scenePos        = viewSpaceToSceneSpace(viewPos, gbufferModelViewInverse);
                vec3 sceneNormal     = decodeNormal(tex1.xy);

                float ao             = compute_hbao(viewPos, coord, normalize(-viewPos), mat3(gbufferModelView) * sceneNormal, bluenoise);
                indirectLight_data.a = ao;
            } else {
                indirectLight_data.a = 1.0;
            }
        }
    #endif

    indirectLight_data = clamp_drawbuffer(indirectLight_data);
}