#version 450 compatibility

/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



#include "/lib/universal.glsl"

/* ------ outputs ------ */
out vec2 coord;     //this is always the topmost variable since it's needed everywhere

flat out float cloudBackscattering;

flat out vec3 cloudLightvector;

flat out mat2x3 lightColor;


/* ------ uniforms ------ */
uniform int worldTime;

uniform float eyeAltitude;

uniform vec2 viewSize;

uniform vec3 sunVector;
uniform vec3 moonVector;

uniform sampler2D colortex4;


/* ------ includes ------ */
#include "/lib/atmosphere/constants.glsl"
#include "/lib/atmosphere/transmittance.glsl"
#include "/lib/atmosphere/project.glsl"

void main() {
    gl_Position     = vec4(gl_Vertex.xy * 2.0 - 1.0, 0.0, 1.0);
    coord           = gl_MultiTexCoord0.xy;

    bool cloudLightsource = worldTime > 23090 || worldTime < 13100;

    lightColor[0]   = cloudLightsource ? calculateAtmosphereTransmittance(sunVector, vec3(0.0, planetRadius + eyeAltitude, 0.0)) * sunIlluminance : calculateAtmosphereTransmittance(moonVector, vec3(0.0, planetRadius + eyeAltitude, 0.0)) * moonIlluminance;
    lightColor[1]   = texture(colortex4, projectSky(vec3(0.0, 1.0, 0.0))).rgb * pi;

    cloudLightvector    = cloudLightsource ? sunVector : moonVector;

    cloudBackscattering = cloudLightsource ? (worldTime > 23000 ? linStep(float(worldTime), 23250.0, 23350.0) : 1.0) * (worldTime > 14000 ? 1.0 : (1.0-linStep(float(worldTime), 12550.0, 12750.0))) : 1.0;
    cloudBackscattering = cloudBackscattering * 0.6 + 0.4;
}