#version 450 compatibility

/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/


#include "/lib/universal.glsl"

/*
const bool colortex0MipmapEnabled = true;
*/

/* ------ framebuffer attachment outputs ------ */
/*DRAWBUFFERS:0345*/
layout(location = 0) out vec4 scenecol_data;
layout(location = 1) out vec4 bokeh_data;
layout(location = 2) out vec4 auxiliary_data1;
layout(location = 3) out vec4 auxiliary_data2;

/* ------ inputs from vertex stage ------ */
in float exposure;

in vec2 coord;

/* ------ uniforms ------ */
uniform int isEyeInWater;

uniform float frameTime;
uniform float blindness;

uniform vec2 viewSize, pixelSize;
uniform vec2 taaOffset;

uniform mat4 gbufferProjectionInverse;

uniform sampler2D colortex0;
uniform sampler2D colortex5;
uniform sampler2D depthtex1;

/* ------ includes ------ */
#include "/lib/atmosphere/constants.glsl"
#include "/lib/fragment/exposure.glsl"

vec3 expFog(in vec3 color, in vec3 viewpos, in vec3 colorFog, in float density) {
	float visibility = saturate(exp(-pow((length(viewpos) * density), 1.5)));
    return mix(colorFog, color, visibility);
}

vec3 screenSpaceToViewSpace(vec3 screenPosition, mat4 projectionInverse) {
	screenPosition = screenPosition * 2.0 - 1.0;

	screenPosition.xy -= taaOffset;

	vec3 viewPosition  = vec3(vec2(projectionInverse[0].x, projectionInverse[1].y) * screenPosition.xy + projectionInverse[3].xy, projectionInverse[3].z);
		 viewPosition /= projectionInverse[2].w * screenPosition.z + projectionInverse[3].w;

	return viewPosition;
}

vec3 bokehImage(float lod, vec2 offset) {
	float r = 0.0;

	vec2 bokehCoord = (coord - offset) * pow(2.0, lod);

	vec3 size = vec3(0.35);
	     size.x *= (1.0 - 1.0 * LENS_SHIFT_AMOUNT);
	     size.y *= (1.0 - 0.5 * LENS_SHIFT_AMOUNT);

	float softness = 0.9;

	for (int i = 0; i < BLADES; i++) {
		float angle = (i/float(BLADES)) * tau;
		vec2 axis = vec2(cos(angle), sin(angle));
		r = max(r, dot(axis, bokehCoord - 0.5));
	}

	vec3 bokeh;
	bokeh = saturate(1.0 - smoothstep(size * softness, size, vec3(r)));

	return bokeh;
}

void main() {
    scenecol_data     = stexLod(colortex0, 0);  //that macro certainly makes it neater
    scenecol_data.a   = 1.0;
    
    float scenedepth1 = stex(depthtex1).r;
    vec3 viewpos1     = screenSpaceToViewSpace(vec3(coord, scenedepth1), gbufferProjectionInverse);

    if(blindness != 0.0)    scenecol_data.rgb = expFog(scenecol_data.rgb, viewpos1, vec3(0.0), 0.4 * blindness);
    if(isEyeInWater == 2.0) scenecol_data.rgb = expFog(scenecol_data.rgb, viewpos1, vec3(1.0, 0.15, 0.0), 0.5);

    vec2 bokehCoord   = (coord - vec2(bokehOffset)) * pow(2.0, LensLOD);
    float bokehAlpha  = float(bokehCoord.x < 1.0 && bokehCoord.y < 1.0 && bokehCoord.x > 0.0 && bokehCoord.y > 0.0);

    if(bokehAlpha <= 0.0) {
        bokeh_data    = vec4(0.0);
    } else {
        bokeh_data    = vec4(bokehImage(LensLOD, vec2(bokehOffset)), bokehAlpha);
    }

    scenecol_data     = make_drawbuffer(scenecol_data);
    auxiliary_data1   = vec4(0.0);
    auxiliary_data2   = vec4(stex(colortex5).rgb, exposure);
}