#version 450 compatibility

/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



#include "/lib/universal.glsl"

/*
const int colortex0Format   = RGBA16F;
const int colortex1Format   = RGBA16;
const int colortex2Format   = RGBA16;
const int colortex3Format   = RGBA16F;
const int colortex4Format   = RGBA16F;
const int colortex5Format   = RGBA16F;
const int colortex6Format   = RGBA8;
const int colortex7Format   = RGBA16F;

const vec4 colortex0ClearColor = vec4(0.0, 0.0, 0.0, 1.0);
const vec4 colortex3ClearColor = vec4(0.0, 0.0, 0.0, 0.0);

const bool colortex5Clear = false;
*/

const int noiseTextureResolution    = 256;


/* ------ framebuffer attachment outputs ------ */
/*DRAWBUFFERS:36*/
layout(location = 0) out vec4 indirectLight_data;
layout(location = 1) out vec4 filtering_data;


/* ------ inputs from vertex stage ------ */
in vec2 coord;     //this is always the topmost variable since it's needed everywhere

#ifdef rsm_enabled
    in vec3 lightVector;
#endif


/* ------ uniforms ------ */
uniform int frameCounter;

uniform float eyeAltitude;
uniform float far, near;
uniform float frameTimeCounter;
uniform float wetness;

uniform vec2 pixelSize, viewSize;
uniform vec2 taaOffset;

uniform vec3 cameraPosition;

uniform mat4 gbufferModelViewInverse, gbufferModelView;
uniform mat4 gbufferProjectionInverse, gbufferProjection;

uniform sampler2D colortex1;
uniform sampler2D colortex4;

uniform sampler2D depthtex0;

uniform sampler2D noisetex;
uniform sampler3D depthtex2;

#if (defined cloud_shadows || defined rsm_enabled)
uniform mat4 shadowModelViewInverse;
#endif

#ifdef rsm_enabled
const int shadowMapResolution   = 2560;     //[512 1024 1536 2048 2560 3072 3584 4096 6144 8192 16384]

const bool colortex7Clear   = false;

uniform vec3 previousCameraPosition;

uniform mat4 shadowModelView;
uniform mat4 shadowProjectionInverse, shadowProjection;
uniform mat4 gbufferPreviousModelView, gbufferPreviousProjection;

uniform sampler2D shadowtex1;
uniform sampler2D shadowcolor0;
uniform sampler2D shadowcolor1;
#endif


/* ------ includes ------ */
#include "/lib/util/encoders.glsl"
#include "/lib/atmosphere/constants.glsl"
#include "/lib/fragment/dither.glsl"
#include "/lib/fragment/lowresCommon.glsl"
#define PD_3DNOISE
#include "/lib/fragment/noise.glsl"
#include "/lib/util/transforms.glsl"


/* ------ functions ------ */

vec3 planetCurvePosition(in vec3 x) {
    return vec3(x.x, length(x + vec3(0.0, planetRadius, 0.0))-planetRadius, x.z);
}

/* ------ ambient occlusion and global illumination ------ */
#include "/lib/lighting/ao.glsl"

#ifdef rsm_enabled
    #include "/lib/util/srgb.glsl"
    #include "/lib/lighting/warp.glsl"
    #include "/lib/lighting/rsm.glsl"

    vec3 getShadowpos(vec3 scenePos, out float cd) {
        vec3 pos    = scenePos + vec3(0.08) * lightVector;

        float a     = length(pos);

            pos     = transMAD(shadowModelView, pos);
            pos     = projMAD3(shadowProjection, pos);
            pos.z  *= 0.2;
            pos.z  -= 12e-4 * saturate(a * rcp(256.0));
        
            cd      = -projMAD3(shadowProjectionInverse, pos).z;

        return pos * 0.5 + 0.5;
    }

    vec3 compute_bouncelight(vec3 scenePos, vec3 sceneNormal, vec2 lmap) {
        float cd    = 0.0;
        vec3 shadowpos = getShadowpos(scenePos, cd);

        return compute_rsm(sceneNormal, shadowpos, scenePos, cd, lmap.y);
    }
#endif

void main() {
    float bluenoise     = dither_bluenoise();

    filtering_data      = vec4(0.0, 0.0, 0.0, 1.0);

    /* ------- ambient occlusion just works, global illumination just works ------ */
    indirectLight_data  = vec4(0.0, 0.0, 0.0, 1.0);

    #if (defined ambient_occlusion_enabled || defined rsm_enabled)
        vec2 aoCoord    = coord * 2.0;
        
        if (clamp(aoCoord, 0.0 - pixelSize, 1.0 + pixelSize) == aoCoord) {
            float depth_3x3     = max_depth3x3(depthtex0, aoCoord, pixelSize * sqrt(2.0));

            if (landMask(depth_3x3)) {
                vec2 coord          = saturate(aoCoord);
                float sceneDepth    = texelFetch(depthtex0, ivec2(coord * viewSize), 0).x;

                vec4 tex1           = texelFetch(colortex1, ivec2(coord * viewSize), 0);

                vec3 viewPos        = screenSpaceToViewSpace(vec3(coord, sceneDepth), gbufferProjectionInverse);
                vec3 scenePos       = viewSpaceToSceneSpace(viewPos, gbufferModelViewInverse);
                vec3 sceneNormal    = decodeNormal(tex1.xy);

                #ifdef ambient_occlusion_enabled
                    float ao        = compute_hbao(viewPos, coord, normalize(-viewPos), mat3(gbufferModelView) * sceneNormal, bluenoise);
                    indirectLight_data.a = ao;
                #endif

                #ifdef rsm_enabled
                    filtering_data.rgb  = sceneNormal * 0.5 + 0.5;
                    vec2 sceneLmap      = decode2x8(tex1.z);
                    float fade          = (1.0 - linStep(length(scenePos), 80.0, 144.0));

                    if (fade > 0.0) {
                        indirectLight_data.rgb  = compute_bouncelight(scenePos, sceneNormal, sceneLmap);
                    }
                #endif
            } else {
                indirectLight_data.a    = 1.0;
            }
        }
    #endif

    filtering_data      = clamp_drawbuffer(filtering_data);
    indirectLight_data  = clamp_drawbuffer(indirectLight_data);
}